"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = exports.generateSchemas = void 0;
const path_1 = __importDefault(require("path"));
const schemaGenerator_1 = require("./schemaGenerator");
const toValidation_1 = require("./toValidation");
const constants_1 = require("./constants");
const util_1 = require("./util");
function generateSchemas(rootDir, outDir, tsconfig) {
    var e_1, _a;
    return __awaiter(this, void 0, void 0, function* () {
        const fileEntries = [];
        try {
            for (var _b = __asyncValues(util_1.walk(rootDir)), _c; _c = yield _b.next(), !_c.done;) {
                const iter = _c.value;
                if (path_1.default.basename(iter) === constants_1.TO_SCHEMA_FILE_NAME) {
                    fileEntries.push(iter);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (fileEntries[0] === undefined) {
            throw new Error('unable to find toJSONSchema.ts');
        }
        const data = schemaGenerator_1.compileExportFile(fileEntries[0]);
        const relPaths = Object.keys(data);
        return Promise.all(relPaths.map(relPath => {
            const defaultConfig = {
                tsconfig,
                path: path_1.default.resolve(path_1.default.dirname(fileEntries[0]), relPath) + '.ts',
                encodeRefs: true,
                additionalProperties: true,
                typeValidationDir: outDir,
            };
            return schemaGenerator_1.generateSchema(defaultConfig, data[relPath]);
        }));
    });
}
exports.generateSchemas = generateSchemas;
function init(srcPath, outPath, tscPath, cb) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            console.dir({ srcPath, schemasOut: util_1.getSchemasPath(outPath) });
            yield util_1.ensureDir(util_1.getSchemasPath(outPath));
            yield util_1.ensureDir(util_1.getValidationCodePath(srcPath));
            yield util_1.initExtras(srcPath);
            yield generateSchemas(srcPath, outPath, tscPath);
            yield toValidation_1.AjvHandler.getInstance().genValidationCode(srcPath, outPath);
        }
        catch (err) {
            cb(err);
        }
    });
}
exports.init = init;
