"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSchema = exports.compileExportFile = void 0;
const tsj = __importStar(require("ts-json-schema-generator"));
const promises_1 = require("fs/promises");
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const constants_1 = require("./constants");
const util_1 = require("./util");
const FunctionTypeFormatter_1 = require("./FunctionTypeFormatter");
function compileExportFile(filePath) {
    const program = typescript_1.default.createProgram([filePath], { allowJs: false });
    const src = program.getSourceFile(filePath);
    if (src === undefined) {
        throw new Error('Missing source file');
    }
    const exported = {};
    typescript_1.default.forEachChild(src, (node) => {
        let typeNames = [];
        if (node.kind === typescript_1.default.SyntaxKind.ExportDeclaration) {
            node.forEachChild((child) => {
                if (child.kind === typescript_1.default.SyntaxKind.NamedExports) {
                    child.forEachChild((namedExport) => {
                        typeNames.push((namedExport['name']['escapedText']));
                    });
                }
                if (typescript_1.default.isTokenKind(child.kind) && child['text']) {
                    exported[child['text']] = typeNames.length > 0 ? [...typeNames] : ['*'];
                    typeNames = [];
                }
            });
        }
    });
    return exported;
}
exports.compileExportFile = compileExportFile;
function generateSchema(config, types) {
    return __awaiter(this, void 0, void 0, function* () {
        const program = tsj.createProgram(config);
        const parser = tsj.createParser(program, config);
        const formatter = tsj.createFormatter(config, (fmt, circularReferenceTypeFormatter) => {
            fmt.addTypeFormatter(new FunctionTypeFormatter_1.FunctionTypeFormatter());
        });
        const generator = new tsj.SchemaGenerator(program, parser, formatter, config);
        const outputPath = schemaOutputPath(config);
        const fullSchema = types.reduce((accum, iter) => {
            const schema = generator.createSchema(iter);
            const definitions = schema['definitions'];
            const refs = !!schema['$ref']
                ? [{ '$ref': schema['$ref'] }]
                : Object.keys(definitions).map(def => ({
                    '$ref': `#/definitions/${def}`,
                }));
            const exported = [].concat(schema['exported'] || [], iter);
            const formattedSchema = {
                '$schema': schema['$schema'],
                anyOf: refs,
                definitions: schema['definitions'],
                exported,
            };
            accum = util_1.mergeDeep(accum, formattedSchema);
            return accum;
        }, {});
        let schemaData;
        try {
            schemaData = JSON.stringify(fullSchema, undefined, 2);
        }
        catch (err) {
            throw err;
        }
        return promises_1.unlink(outputPath)
            .catch(err => (err.code === 'ENOENT')
            ? Promise.resolve()
            : Promise.reject(err))
            .then(() => promises_1.writeFile(schemaOutputPath(config), schemaData));
    });
}
exports.generateSchema = generateSchema;
function schemaOutputPath(config) {
    const filePath = config.path;
    const outputDir = util_1.getSchemasPath(config['typeValidationDir']);
    const targetFileName = path_1.default.basename(filePath, path_1.default.extname(filePath)) + constants_1.SCHEMA_SUFFIX;
    return path_1.default.join(outputDir, targetFileName);
}
