"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AjvHandler = void 0;
const ajv_1 = __importDefault(require("ajv"));
const promises_1 = require("fs/promises");
const path_1 = __importDefault(require("path"));
const standalone_1 = __importDefault(require("ajv/dist/standalone"));
const constants_1 = require("./constants");
const util_1 = require("./util");
const tsGenerator_1 = require("./tsGenerator");
class AjvHandler {
    constructor() {
        this.mAjv = new ajv_1.default({
            strict: 'log',
            code: { source: true },
            allErrors: true,
            verbose: true,
            keywords: AjvHandler._customKeywords,
            removeAdditional: false,
        });
    }
    static getInstance() {
        if (!this._instance) {
            this._instance = new this();
        }
        return this._instance;
    }
    genValidationCode(srcPath, outPath) {
        var e_1, _a;
        return __awaiter(this, void 0, void 0, function* () {
            const schemas = [];
            try {
                for (var _b = __asyncValues(util_1.walk(util_1.getSchemasPath(outPath))), _c; _c = yield _b.next(), !_c.done;) {
                    const iter = _c.value;
                    if (iter.endsWith(constants_1.SCHEMA_SUFFIX)) {
                        try {
                            const data = yield promises_1.readFile(iter, { encoding: 'utf-8' });
                            const schema = JSON.parse(data);
                            const refs = Object.keys(schema['definitions']);
                            const valModuleFile = path_1.default.basename(iter)
                                .replace(constants_1.SCHEMA_SUFFIX, constants_1.VALIDATION_FILE_SUFFIX);
                            const outputPath = path_1.default.join(util_1.getValidationCodePath(srcPath), valModuleFile);
                            schemas.push({ schema, filePath: iter, refs, validateFilePath: outputPath });
                        }
                        catch (err) {
                            console.error(err);
                        }
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            const tsCodeGenerator = tsGenerator_1.ValidationFileGenerator.getInstance(path_1.default.join(util_1.getValidationCodePath(srcPath), 'validation.ts'));
            for (const inst of schemas) {
                try {
                    const validate = this.mAjv.compile(inst.schema);
                    const moduleCode = standalone_1.default(this.mAjv, validate);
                    const filtered = moduleCode.replace(/\"ajv\/[^)]*/gm, '"./dummy"');
                    yield promises_1.writeFile(inst.validateFilePath, filtered);
                    tsCodeGenerator.addSchema(inst);
                }
                catch (err) {
                    console.error(err);
                }
            }
            tsCodeGenerator.genValidationFile();
        });
    }
}
exports.AjvHandler = AjvHandler;
AjvHandler._customKeywords = [
    { keyword: 'exported', type: 'array' }
];
