"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidationFileGenerator = void 0;
const path_1 = __importDefault(require("path"));
const promises_1 = __importDefault(require("fs/promises"));
const ts = __importStar(require("typescript"));
const shortId = __importStar(require("shortid"));
class ValidationFileGenerator {
    constructor(outputPath) {
        this.mOutputFilePath = outputPath;
        this.mSchemas = [];
    }
    static getId() {
        return ValidationFileGenerator.id++;
    }
    static getInstance(outputPath) {
        if (!this._instance) {
            this._instance = new this(outputPath);
        }
        if (this._instance.mOutputFilePath !== outputPath) {
            this._instance.mOutputFilePath = outputPath;
        }
        return this._instance;
    }
    addSchema(schema) {
        if (schema) {
            const id = shortId.generate().replace(/[^a-zA-Z ]/g, '');
            this.mSchemas.push(Object.assign(Object.assign({}, schema), { id }));
        }
    }
    genValidationFile() {
        return __awaiter(this, void 0, void 0, function* () {
            const file = ts.createSourceFile(this.mOutputFilePath, '', ts.ScriptTarget.ESNext, false, ts.ScriptKind.TS);
            const printer = ts.createPrinter({ newLine: ts.NewLineKind.LineFeed });
            this.genImportNodes();
            const importNodes = this.mSchemas.map(schema => { var _a; return (_a = schema.importNode) === null || _a === void 0 ? void 0 : _a.importStatement; });
            const res = importNodes.map(node => printer.printNode(ts.EmitHint.Unspecified, node, file)).join('');
            const functionNodes = this.mSchemas.reduce((accum, iter) => {
                const validationFunctions = this.genValidateFunctionNodes(iter)
                    .filter(funcNode => accum.find(funcDecl => { var _a, _b; return ((_a = funcDecl.name) === null || _a === void 0 ? void 0 : _a.escapedText) === ((_b = funcNode.name) === null || _b === void 0 ? void 0 : _b.escapedText); }) === undefined);
                accum = accum.concat(validationFunctions);
                return accum;
            }, []);
            const res2 = functionNodes.map(node => printer.printNode(ts.EmitHint.Unspecified, node, file)).join('');
            yield promises_1.default.writeFile(this.mOutputFilePath, ''.concat(res, res2), { encoding: 'utf8' });
        });
    }
    genImportNodes() {
        this.mSchemas = this.mSchemas.reduce((accum, iter) => {
            const relPath = path_1.default.relative(path_1.default.dirname(this.mOutputFilePath), iter.validateFilePath);
            const importTarget = './' + path_1.default.join(path_1.default.dirname(relPath), path_1.default.basename(relPath, path_1.default.extname(relPath)));
            const importIdentifier = ts.factory.createIdentifier(`validate${iter.id}`);
            const importStatement = ts.factory.createImportDeclaration(undefined, undefined, ts.factory.createImportClause(false, importIdentifier, undefined), ts.factory.createStringLiteral(importTarget));
            accum.push(Object.assign(Object.assign({}, iter), { importNode: {
                    importIdentifier,
                    importStatement,
                } }));
            return accum;
        }, []);
    }
    genValidateFunctionNodes(schema) {
        const { factory } = ts;
        const functionDeclarations = schema.refs.reduce((accum, iter) => {
            var _a;
            if (!!(schema === null || schema === void 0 ? void 0 : schema.schema['exported']) && !(schema === null || schema === void 0 ? void 0 : schema.schema['exported'].includes(iter))) {
                return accum;
            }
            const functionName = factory.createIdentifier(`validate${iter}`);
            const dataIdentifier = factory.createIdentifier('data');
            const dataParameter = factory.createParameterDeclaration(undefined, undefined, undefined, dataIdentifier);
            const importIdentifier = (_a = schema.importNode) === null || _a === void 0 ? void 0 : _a.importIdentifier;
            const callExpression = factory.createCallExpression(importIdentifier, undefined, [dataIdentifier]);
            const resultIdentifier = factory.createIdentifier('res');
            const resVar = factory.createVariableDeclaration(resultIdentifier, undefined, undefined, callExpression);
            const returnStatement = factory.createReturnStatement(factory.createConditionalExpression(factory.createParenthesizedExpression(factory.createBinaryExpression(resultIdentifier, factory.createToken(ts.SyntaxKind.EqualsEqualsEqualsToken), factory.createFalse())), factory.createToken(ts.SyntaxKind.QuestionToken), factory.createPropertyAccessExpression(factory.createPropertyAccessExpression(factory.createPropertyAccessExpression(importIdentifier, factory.createIdentifier("prototype")), factory.createIdentifier("constructor")), factory.createIdentifier("errors")), factory.createToken(ts.SyntaxKind.ColonToken), factory.createArrayLiteralExpression([], false)));
            accum.push(factory.createFunctionDeclaration(undefined, [factory.createToken(ts.SyntaxKind.ExportKeyword)], undefined, functionName, undefined, [dataParameter], factory.createArrayTypeNode(factory.createKeywordTypeNode(ts.SyntaxKind.AnyKeyword)), factory.createBlock([factory.createVariableStatement(undefined, [resVar]), returnStatement], true)));
            return accum;
        }, []);
        return functionDeclarations;
    }
    compile() {
        const options = {
            noEmitOnError: true,
            noImplicitAny: false,
            target: ts.ScriptTarget.ES5,
            module: ts.ModuleKind.CommonJS
        };
        const program = ts.createProgram([this.mOutputFilePath], options);
        const emitResult = program.emit();
        const allDiagnostics = ts.getPreEmitDiagnostics(program)
            .concat(emitResult.diagnostics);
        allDiagnostics.forEach(diagnostic => {
            if (diagnostic.file) {
                let { line, character } = ts.getLineAndCharacterOfPosition(diagnostic.file, diagnostic.start);
                let message = ts.flattenDiagnosticMessageText(diagnostic.messageText, "\n");
                console.log(`${diagnostic.file.fileName} (${line + 1},${character + 1}): ${message}`);
            }
            else {
                console.log(ts.flattenDiagnosticMessageText(diagnostic.messageText, "\n"));
            }
        });
    }
}
exports.ValidationFileGenerator = ValidationFileGenerator;
ValidationFileGenerator.id = 0;
