"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {
    var i, p;
    return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
    function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
};
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeDeep = exports.getValidationCodePath = exports.getSchemasPath = exports.initExtras = exports.ensureDir = exports.walk = void 0;
const path_1 = __importDefault(require("path"));
const promises_1 = __importDefault(require("fs/promises"));
function walk(rootPath) {
    return __asyncGenerator(this, arguments, function* walk_1() {
        const directories = yield __await(promises_1.default.readdir(rootPath, { withFileTypes: true }));
        for (const dir of directories) {
            const fullPath = path_1.default.resolve(rootPath, dir.name);
            if (dir.isDirectory()) {
                yield __await(yield* __asyncDelegator(__asyncValues(walk(fullPath))));
            }
            else {
                yield yield __await(fullPath);
            }
        }
    });
}
exports.walk = walk;
function ensureDir(dirPath) {
    return __awaiter(this, void 0, void 0, function* () {
        return promises_1.default.mkdir(dirPath, { recursive: true })
            .then(() => Promise.resolve())
            .catch(err => ['EEXIST'].includes(err.code)
            ? Promise.resolve()
            : Promise.reject(err));
    });
}
exports.ensureDir = ensureDir;
function initExtras(outPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const srcDummyFile = path_1.default.join(__dirname, 'dummy.js');
        const destDummyFile = path_1.default.join(getValidationCodePath(outPath), 'dummy.js');
        try {
            yield promises_1.default.copyFile(srcDummyFile, destDummyFile);
        }
        catch (err) {
            return Promise.reject(err);
        }
    });
}
exports.initExtras = initExtras;
function getSchemasPath(outPath) {
    return path_1.default.join(outPath, 'generatedSchemas');
}
exports.getSchemasPath = getSchemasPath;
function getValidationCodePath(outPath) {
    return path_1.default.join(outPath, 'validationCode');
}
exports.getValidationCodePath = getValidationCodePath;
function mergeDeep(...objects) {
    const isObject = obj => obj && typeof obj === 'object';
    return objects.reduce((prev, obj) => {
        Object.keys(obj).forEach(key => {
            const pVal = prev[key];
            const oVal = obj[key];
            if (Array.isArray(pVal) && Array.isArray(oVal)) {
                prev[key] = pVal.concat(...oVal);
            }
            else if (isObject(pVal) && isObject(oVal)) {
                prev[key] = mergeDeep(pVal, oVal);
            }
            else {
                prev[key] = oVal;
            }
        });
        return prev;
    }, {});
}
exports.mergeDeep = mergeDeep;
